@@ -16,6 +16,37 @@
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ##=============================================================================
 
+
+GECODE_VERSION = None
+SRCDIR = None
+
+def gecode_version():
+    global GECODE_VERSION
+    if GECODE_VERSION is not None:
+        return GECODE_VERSION
+    from distutils.ccompiler import new_compiler, customize_compiler
+    import os
+    cxx = new_compiler()
+    customize_compiler(cxx)
+    file_hh = "_gecode_version.hh"
+    file_txt = "_gecode_version.txt"
+    f = file(file_hh,"w")
+    f.write("""#include "gecode/support/config.hpp"
+@@GECODE_VERSION""")
+    f.close()
+    cxx.preprocess(file_hh,output_file=file_txt)
+    f = open(file_txt)
+    version = ""
+    for line in f:
+        if line.startswith("@@"):
+            version = line[3:-2]
+            break
+    f.close()
+    os.remove(file_hh)
+    os.remove(file_txt)
+    GECODE_VERSION = version
+    return version
+
 import re
 
 class Type(object):
@@ -572,83 +603,25 @@ class OStream(object):
             self.write(' ')
             n -= 1
 
-class IntRelType(object):
-    TYPE = 'IntRelType'
-    ENUM = ('IRT_EQ','IRT_NQ','IRT_LQ','IRT_LE','IRT_GQ','IRT_GR')
+ENUM_CLASSES = None
+ENUM_CLASSES_AVOID = ('ScriptMode','ViewSelStatus','ExecStatus',
+                      'ActorProperty','SpaceStatus')
 
-class BoolOpType(object):
-    TYPE = 'BoolOpType'
-    ENUM = ('BOT_AND','BOT_OR','BOT_IMP','BOT_EQV','BOT_XOR')
+def enum_classes():
+    global ENUM_CLASSES
+    if ENUM_CLASSES is None:
+        filename = "gecode-enums-%s.py" % gecode_version()
+        if SRCDIR is not None:
+            import os.path
+            filename = os.path.join(SRCDIR,filename)
+        import imp
+        ENUM_CLASSES = imp.load_source(
+            "gecode_enums",
+            filename).ENUM_CLASSES
+        ENUM_CLASSES = tuple(x for x in ENUM_CLASSES if x.TYPE not in ENUM_CLASSES_AVOID)
+    return ENUM_CLASSES
 
-class IntConLevel(object):
-    TYPE = 'IntConLevel'
-    ENUM = ('ICL_VAL','ICL_BND','ICL_DOM','ICL_DEF')
-
-class SetRelType(object):
-    TYPE = 'SetRelType'
-    ENUM = ('SRT_EQ','SRT_NQ','SRT_SUB','SRT_SUP','SRT_DISJ','SRT_CMPL')
-
-class SetOpType(object):
-    TYPE = 'SetOpType'
-    ENUM = ('SOT_UNION','SOT_DUNION','SOT_INTER','SOT_MINUS')
-
-class IntVarBranch(object):
-    TYPE = 'IntVarBranch'
-    ENUM = ('INT_VAR_NONE','INT_VAR_RND','INT_VAR_DEGREE_MIN',
-            'INT_VAR_DEGREE_MAX','INT_VAR_AFC_MIN','INT_VAR_AFC_MAX',
-            'INT_VAR_MIN_MIN','INT_VAR_MIN_MAX','INT_VAR_MAX_MIN',
-            'INT_VAR_MAX_MAX','INT_VAR_SIZE_MIN','INT_VAR_SIZE_MAX',
-            'INT_VAR_SIZE_DEGREE_MIN','INT_VAR_SIZE_DEGREE_MAX',
-            'INT_VAR_SIZE_AFC_MIN','INT_VAR_SIZE_AFC_MAX',
-            'INT_VAR_REGRET_MIN_MIN','INT_VAR_REGRET_MIN_MAX',
-            'INT_VAR_REGRET_MAX_MIN','INT_VAR_REGRET_MAX_MAX')
-
-class IntValBranch(object):
-    TYPE = 'IntValBranch'
-    ENUM = ('INT_VAL_MIN','INT_VAL_MED','INT_VAL_MAX',
-            'INT_VAL_RND','INT_VAL_SPLIT_MIN','INT_VAL_SPLIT_MAX',
-            'INT_VAL_RANGE_MIN','INT_VAL_RANGE_MAX',
-            'INT_VALUES_MIN','INT_VALUES_MAX')
-
-class SetVarBranch(object):
-    TYPE = 'SetVarBranch'
-    ENUM = ('SET_VAR_NONE','SET_VAR_RND','SET_VAR_DEGREE_MIN',
-            'SET_VAR_DEGREE_MAX','SET_VAR_AFC_MIN','SET_VAR_AFC_MAX',
-            'SET_VAR_MIN_MIN','SET_VAR_MIN_MAX','SET_VAR_MAX_MIN',
-            'SET_VAR_MAX_MAX','SET_VAR_SIZE_MIN','SET_VAR_SIZE_MAX',
-            'SET_VAR_SIZE_DEGREE_MIN','SET_VAR_SIZE_DEGREE_MAX',
-            'SET_VAR_SIZE_AFC_MIN','SET_VAR_SIZE_AFC_MAX')
-
-class SetValBranch(object):
-    TYPE = 'SetValBranch'
-    ENUM = ('SET_VAL_MIN_INC','SET_VAL_MIN_EXC','SET_VAL_MED_INC',
-            'SET_VAL_MED_EXC','SET_VAL_MAX_INC','SET_VAL_MAX_EXC',
-            'SET_VAL_RND_INC','SET_VAL_RND_EXC')
-
-class TaskType(object):
-    TYPE = 'TaskType'
-    ENUM = ('TT_FIXP','TT_FIXS','TT_FIXE')
-
-class ExtensionalPropKind(object):
-    TYPE = 'ExtensionalPropKind'
-    ENUM = ('EPK_DEF','EPK_SPEED','EPK_MEMORY')
-
-class IntAssign(object):
-    TYPE = 'IntAssign'
-    ENUM = ('INT_ASSIGN_MIN', 'INT_ASSIGN_MED', 'INT_ASSIGN_MAX', 'INT_ASSIGN_RND')
-
-class SetAssign(object):
-    TYPE = 'SetAssign'
-    ENUM = ('SET_ASSIGN_MIN_INC', 'SET_ASSIGN_MIN_EXC', 'SET_ASSIGN_MED_INC',
-            'SET_ASSIGN_MED_EXC', 'SET_ASSIGN_MAX_INC', 'SET_ASSIGN_MAX_EXC',
-            'SET_ASSIGN_RND_INC', 'SET_ASSIGN_RND_EXC')
-
-ENUM_CLASSES = (IntRelType, BoolOpType, IntConLevel,
-                SetRelType, SetOpType, IntVarBranch,
-                IntValBranch, SetVarBranch, SetValBranch,
-                TaskType, ExtensionalPropKind, IntAssign, SetAssign)
-
-for e in ENUM_CLASSES:
+for e in enum_classes():
     class C(CythonEnum, e): pass
     VARHANDLERS[e.TYPE] = C
 
@@ -667,7 +640,7 @@ class CythonEnumImplGenerator(object):
 
     def generate(self):
         print 'cdef extern from "gecode-common.icc" namespace "Gecode":'
-        for c in ENUM_CLASSES:
+        for c in enum_classes():
             class C(c,CythonEnumImpl): pass
             o = C()
             o.generate()
@@ -683,36 +656,16 @@ class CythonEnumWrap(object):
 class CythonEnumWrapGenerator(object):
 
     def generate(self):
-        for c in ENUM_CLASSES:
+        for c in enum_classes():
             class C(c,CythonEnumWrap): pass
             o = C()
             o.generate()
 
-def gecode_version():
-    from distutils.ccompiler import new_compiler, customize_compiler
-    import os
-    cxx = new_compiler()
-    customize_compiler(cxx)
-    file_hh = "_gecode_version.hh"
-    file_txt = "_gecode_version.txt"
-    f = file(file_hh,"w")
-    f.write("""#include "gecode/support/config.hpp"
-@@GECODE_VERSION""")
-    f.close()
-    cxx.preprocess(file_hh,output_file=file_txt)
-    f = open(file_txt)
-    version = ""
-    for line in f:
-        if line.startswith("@@"):
-            version = line[3:-2]
-            break
-    f.close()
-    os.remove(file_hh)
-    os.remove(file_txt)
-    return version
-
 def generate_files():
     filename = "gecode-prototypes-%s.hh" % gecode_version()
+    if SRCDIR is not None:
+        import os.path
+        filename = os.path.join(SRCDIR,filename)
     stdout = sys.stdout
     try:
         sys.stdout = file("gecode_python_enum.pxi","w")
